"use client";

import { useId } from "react";

import { Controller } from "react-hook-form";

import { cn } from "@/lib/utils";

import { InputFieldProps } from "../types/all-fields";

type InputVariant =
	| "default"
	| "outline"
	| "ghost"
	| "transparent"
	| "under_line";
type InputSize = "sm" | "md" | "lg";

const variantStyles: Record<InputVariant, string> = {
	default: "hover:bg-accent border border-input bg-background",
	outline: "border-2 border-gray-300 bg-transparent",
	ghost: "border-none bg-gray-100",
	transparent: "border-none bg-transparent",
	under_line: "rounded-none bg-transparent  border-b-[1px] border-primary",
};

const sizeStyles: Record<InputSize, string> = {
	sm: "px-2 py-1 text-sm",
	md: "px-3 py-2 text-base",
	lg: "px-4 py-3 text-lg",
};

const InputField = <T,>({
	name,
	control,
	label,
	required = false,
	type = "text",
	containerClassName,
	inputClassName,
	labelClassName,
	errorClassName,
	onChange,
	variant = "default",
	size = "md",
	hidden = false,
	inputProps = {},
	prefix,
	suffix,
	placeholder,
	color,
}: InputFieldProps<T>) => {
	const id = useId();
	return (
		<Controller
			name={name}
			control={control}
			render={({ field, fieldState: { error } }) => (
				<div
					className={cn(
						"col-span-12 p-2",
						{ hidden: hidden },
						containerClassName
					)}
				>
					<label
						htmlFor={id}
						className={cn(
							"mb-1 block text-base font-medium text-gray-700",
							color && `text-[${color}]`,
							labelClassName
						)}
					>
						{label}
						{required && <span className="ml-1 text-red-500">{"*"}</span>}
					</label>

					<div className="relative flex items-center">
						{prefix && (
							<div className="absolute z-10 flex items-center ltr:left-3 rtl:right-3">
								{prefix}
							</div>
						)}
						<input
							id={id}
							type={type}
							className={cn(
								"w-full rounded-md transition-colors",
								sizeStyles[size],
								"hover:text-accent-foreground focus:outline-none",
								color && `text-[${color}] hover:text-[${color}]`,
								{
									"border-red-500 focus:ring-red-500": !!error,
									"ltr:pl-10 rtl:pr-10": !!prefix,
									"ltr:pr-10 rtl:pl-10": !!suffix,
								},
								variantStyles[variant],
								inputClassName
							)}
							placeholder={placeholder}
							{...field}
							onChange={(e) => {
								if (onChange) {
									onChange(e.target.value);
								} else {
									field.onChange(e);
								}
							}}
							{...inputProps}
						/>
						{suffix && (
							<div className="absolute z-10 flex items-center ltr:right-3 rtl:left-3 rtl:right-auto">
								{suffix}
							</div>
						)}
					</div>
					{error && (
						<span
							className={cn("-bottom-6 text-sm text-red-500", errorClassName)}
						>
							{error.message}
						</span>
					)}
				</div>
			)}
		/>
	);
};

export default InputField;
